<?php
/************************************************************
DBF reader Class v0.04  by Faro K Rasyid (Orca)
orca75_at_dotgeek_dot_org
v0.05 by Nicholas Vrtis
vrtis_at_vrtisworks_dot_com
1) changed to not read in complete file at creation.
2) added function to read individual rows
3) added support for Memo fields in dbt files.
4) See: http://www.clicketyclick.dk/databases/xbase/format/dbf.html#DBF_STRUCT
   for some additional information on XBase structure...
5) NOTE: the whole file (and the memo file) is read in at once.  So this could
   take a lot of memory for large files.

v0.06 by janusz dobrowolski 
janusz@iron.form.pl
1) Added support for FoxPro memo files
2) Added support for floats (dec)

Input		: name of the DBF( dBase III plus) file
Output	:	- dbf_num_rec, the number of records
			- dbf_num_field, the number of fields
			- dbf_names, array of field information ('name', 'len', 'type','dec')

Usage	example:
$file= "your_file.dbf";//WARNING !!! CASE SENSITIVE APPLIED !!!!!
$dbf = new dbf_class($file);
$num_rec=$dbf->dbf_num_rec;
$num_field=$dbf->dbf_num_field;

for($i=0; $i<$num_rec; $i++){
    $row = $dbf->getRow($i);
	for($j=0; $j<$num_field; $j++){
		echo $row[$j].' ');
	}
	echo('<br>');
}

Thanks to :
- Willy
- Miryadi

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
See the GNU  Lesser General Public License for more details.
  
**************************************************************/ 

if ($_SESSION['language']->encoding == 'iso-8859-2' && $import_cfg['recode']) {
	include("mazovia.php"); // for Polish MSDos FoxPro version
}

class dbf_class {
		
    var $dbf_num_rec;           //Number of records in the file
    var $dbf_num_field;         //Number of columns in each row
    var $dbf_names = array();   //Information on each column ['name'],['len'],['type'],['dec']
    //These are private....
    var $_raw;               //The raw input file
    var $_rowsize;           //Length of each row
    var $_hdrsize;           //Length of the header information (offset to 1st record)
    var $_memos;             //The raw memo file (if there is one).

    function dbf_class($filename) {
        if ( !file_exists($filename)) {
            display_error(_("Selected DBF file doesn't exists: ").$filename); return;
        }
        $tail=substr($filename,-4);
        if (strcasecmp($tail, '.dbf')!=0) {
            display_error(_('Invalid DBF file name extension')); return;
        }
				
        //Read the File
        $handle = fopen($filename, "r");
        if (!$handle) { 
			display_error(_("Cannot open DBF file")); return; }
        $filesize = filesize($filename);
        $this->_raw = fread ($handle, $filesize);
        fclose ($handle);
        //Make sure that we indeed have a dbf file...
        if(!(ord($this->_raw[0]) == 3 || ord($this->_raw[0]) == 131 || ord($this->_raw[0]) == 0xf5) && ord($this->_raw[$filesize]) != 26) {
            display_error(_('Selected DBF file is corrupted'));
        }
        // 3= file without DBT memo file; 131 ($83)= file with a DBT.
	// 0x3f => fox with .fpt file
        $arrHeaderHex = array();
        for($i=0; $i<32; $i++){
            $arrHeaderHex[$i] = str_pad(dechex(ord($this->_raw[$i]) ), 2, "0", STR_PAD_LEFT);
        }
        //Initial information
        $line = 32;//Header Size
        //Number of records
        $this->dbf_num_rec=  hexdec($arrHeaderHex[7].$arrHeaderHex[6].$arrHeaderHex[5].$arrHeaderHex[4]);
        $this->_hdrsize= hexdec($arrHeaderHex[9].$arrHeaderHex[8]);//Header Size+Field Descriptor
        //Number of fields
        $this->_rowsize = hexdec($arrHeaderHex[11].$arrHeaderHex[10]);
		$this->dbf_num_field = floor(($this->_hdrsize - $line ) / $line ) ;//Number of Fields
				
        //Field properties retrieval looping
        for($j=0; $j<$this->dbf_num_field; $j++){
            $name = '';
            $beg = $j*$line+$line;
            for($k=$beg; $k<$beg+11; $k++){
                if(ord($this->_raw[$k])!=0){
                    $name .= $this->_raw[$k];
                }
            }
            $this->dbf_names[$j]['name']= strtolower($name);//Name of the Field
            $this->dbf_names[$j]['len']= ord($this->_raw[$beg+16]);//Length of the field
            $this->dbf_names[$j]['dec']= ord($this->_raw[$beg+17]);//Decimal points
            $this->dbf_names[$j]['type']= $this->_raw[$beg+11];
            $this->dbf_names[strtolower($name)] = $this->dbf_names[$j]; // also assoc
        }
        if (ord($this->_raw[0])==131) { //See if this has a memo file with it...
            //Read the File
            $tail=substr($tail,-1,1);   //Get the last character...
            if ($tail=='F'){            //See if upper or lower case
                $tail='T';              //Keep the case the same
            } else {
                $tail='t';
            }
            $memoname = substr($filename,0,strlen($filename)-1).$tail;
            $handle = fopen($memoname, "r");
            if (!$handle) { display_error(_("Cannot read DBT file")); return; }
            $filesize = filesize($memoname);
            $this->_memos = fread ($handle, $filesize);
            fclose ($handle);
        }
	 else
        if (ord($this->_raw[0])==0xf5) { // FoxPro memo format...
            //Read the File
            $tail=substr($tail,-3,3);   //Get the last character...
            if ($tail[2]=='F'){            //See if upper or lower case
                $tail='FPT';              //Keep the case the same
            } else {
                $tail='fpt';
            }
            $memoname = substr($filename,0,strlen($filename)-3).$tail;
            $handle = fopen($memoname, "r");
            if (!$handle) { display_error(_("Cannot read FPT file")); return; }
            $filesize = filesize($memoname);
            $this->_memos = fread ($handle, $filesize);
            fclose ($handle);
        }
    }
    
    function getRow($recnum, $all=false) {

   global $recode;

        if(!isset($recode))  {  $recode= array();}

        $memoeot = chr(26).chr(26);
        $rawrow = substr($this->_raw,$recnum*$this->_rowsize+$this->_hdrsize,$this->_rowsize);
        $rowrecs = array();
        $beg=1;
        if (ord($rawrow[0])==0x2a && !$all) {
            return false;   //Record is deleted...
        }
        for ($i=0; $i<$this->dbf_num_field; $i++) {
            $col = trim(substr($rawrow, $beg, $this->dbf_names[$i]['len']));
            if ($this->dbf_names[$i]['type'] != 'M') {
			    $rowrecs[]=strtr($col,$recode);
            } else {
	    		if(ord($this->_raw[0]) == 0xf5) { // FoxPro
	       			$blksize = ord($this->_memos[6])*256+ord($this->_memos[7]);
            		$memobeg=$col*$blksize;
					if(!$memobeg) { // empty memo field
						$rowrecs[] ='';
					}
					else {
						$flen = 0;
						for($j = 4; $j < 8; $j++) {
			    			$flen = $flen*256+ord($this->_memos[$memobeg+$j]);
						}
            			$rowrecs[]=strtr(substr($this->_memos,$memobeg+8,$flen),$recode);
					}
	      		} else {
                	$memobeg=$col*512;  //Find start of the memo block (0=header so it works)
                	$memoend=strpos($this->_memos,$memoeot,$memobeg);   //Find the end of the memo
                	$rowrecs[]=strtr(substr($this->_memos,$memobeg,$memoend-$memobeg),$recode);
              	}
	    }
            $beg += $this->dbf_names[$i]['len'];
        }
	return $rowrecs;
    }
    
    function getRowAssoc($recnum, $all=false) {
	   	global $recode;

        if(!isset($recode))  {  $recode= array();}

        $memoeot = chr(26).chr(26);
        $rawrow = substr($this->_raw,$recnum*$this->_rowsize+$this->_hdrsize,$this->_rowsize);
        $rowrecs = array();
        $beg=1;
        if (ord($rawrow[0])==0x2a && !$all) {
            return false;   //Record is deleted...
        }
        for ($i=0; $i<$this->dbf_num_field; $i++) {
			$flddef = $this->dbf_names[$i];
            $col = trim(substr($rawrow, $beg, $flddef['len']));
            if ($flddef['type'] != 'M') {
			    $rowrecs[$flddef['name']]=strtr($col,$recode);
            } else {
	    		if(ord($this->_raw[0]) == 0xf5) { // FoxPro
	       			$blksize = ord($this->_memos[6])*256+ord($this->_memos[7]);
            		$memobeg=$col*$blksize;
					if(!$memobeg) { // empty memo field
						$rowrecs[$flddef['name']] ='';
					}
					else {
						$flen = 0;
						for($j = 4; $j < 8; $j++) {
			    			$flen = $flen*256+ord($this->_memos[$memobeg+$j]);
						}
            			$rowrecs[$flddef['name']]=strtr(substr($this->_memos,$memobeg+8,$flen),$recode);
					}
	      		} else {
                	$memobeg=$col*512;  //Find start of the memo block (0=header so it works)
                	$memoend=strpos($this->_memos,$memoeot,$memobeg);   //Find the end of the memo
                	$rowrecs[$flddef['name']]=strtr(substr($this->_memos,$memobeg,$memoend-$memobeg),$recode);
              	}
	    }
            $beg+=$flddef['len'];
        }
	return $rowrecs;
	}
}//End of Class
?>
