<?php
/*
	Import editor stores current import configuration in
	company/x/.../import_cfg.php file.
*/
define('IMPORT_DIR', $comp_path.'/'.user_company().'/import');
define('IMPORT_CFG', 'import_cfg.php');

get_text::add_domain($lang->code, $path_to_root.'/modules/import/lang');
$import_cfg =& read_config(get_post('datadir'));

//-----------------------------------------------------------------------------
//	Checks module installation and creates working subdirectory on first run.
//
function chk_module_install()  
{

	if (!file_exists(IMPORT_DIR)) {
		// seems this is first call of just installed import module
		mkdir(IMPORT_DIR);
		mkdir(IMPORT_DIR.'/example');
		// copy example data
		$dir = opendir('./example');
		while (($fname = readdir($dir)) !== false) {
			if ($fname=='.' || $fname=='..') continue;
    		@copy('./example/'.$fname, IMPORT_DIR.'/example/'.$fname);
		}
		display_notification(_('FoxPro/xBase import module has been initialized succesfully.')
			.'<br>'._('You will find sample configuration in \'example\' subdirectory'));
  	}
}

//-----------------------------------------------------------------------------
//	Returns string prepared to store in php single-quoted notation
// ( \ and ' escaped)
function escape($eval)
{
	return	preg_replace("{(['\\\\])}","\\\\$1",$eval);
}

//-----------------------------------------------------------------------------
//
function save_config($datadir) 
{
	global $import_cfg;
	
	$cfgname = IMPORT_DIR.'/'.$datadir.'/'.IMPORT_CFG;
	
	$file = @fopen($cfgname, 'w');
	if(!$file) {
		display_error(_('Cannot open import config file').' '.IMPORT_CFG);
		return;
	}
	$ret = !@fwrite( $file, 
"<?php
//
// FoxPro/dBase import definitions
// This file is updated whenever 'Process' or 'Save' button is pressed
// at import module main page.

\$import_cfg = array(
// Directory with source data files
	'data_dir' => '{$import_cfg['data_dir']}',
// When set to true all previous data in updated tables are lost!
	'flush' => ".($import_cfg['flush'] ? 'true':'false').",
	'recode' => ".($import_cfg['recode'] ? 'true':'false').",
	'tables' => array(");

	foreach($import_cfg['tables'] as $tblname => $table) {
		if (!@count($table['fields']) && @empty($table['filter'])) continue;
		$ret |= !@fwrite($file, "
		'$tblname' => array(
			'fname' => '{$table['fname']}',
			'name'  => '$tblname',
			'filter' => '". escape($table['filter'])."',
			'getdeleted' => ".($table['getdeleted'] ? 'true':'false').",
			'fields' => array(");
	if (count($table['fields']))
		foreach($table['fields'] as $name => $def) {
		 if( is_array($def)) {
			$ret |= !@fwrite($file, "
				'$name' => array("
				 ."'eval'=>'". escape($def['eval'])."'),");
		 } else {
			$ret |= !@fwrite($file, "
			'$name' => '$def',");
		 }
		}
		$ret |= !@fwrite($file, "\n 			),\n		),");
	}

	$ret |= !@fwrite($file, "\n	)\n);\n?>");
	$ret |= !@fclose($file);
	if ($ret)
		display_error(_("Error while writing FoxPro/dBase import configuration"));
	else
		display_notification(_("FoxPro/dBase import configuration has been saved"));
}
//-----------------------------------------------------------------------------
//
//
function &read_config($datadir='') 
{
	global $import_cfg;
	
	chk_module_install();
	if ($datadir == '') {	// last used or first
		if (!isset($_SESSION['import_cfg'])) {
			$dirs = get_import_dirs();
			$datadir = current($dirs);
		}
		 else
			$datadir = $_SESSION['import_cfg']['data_dir'];	
	}
	if (!isset($_SESSION['import_cfg']) || 
		$_SESSION['import_cfg']['data_dir'] != $datadir) {
		$cfgname = IMPORT_DIR.'/'.$datadir.'/'.IMPORT_CFG;

		@include($cfgname);
		if (!isset($import_cfg)) {	// no file found
			$import_cfg = array(
				'data_dir' => $datadir,
				'flush' => false,
				'recode' => false,
				'tables' => array()
			);
			save_config($datadir);		// store default
		}
		$_SESSION['import_cfg'] = $import_cfg;
	}
	return $_SESSION['import_cfg'];
}

//-----------------------------------------------------------------------------
//
//	Evaluation of $eval expression in environment with 
//	current row fields content moved to local variables.
//
function eval_sandbox($eval, $row)
{
	global $messages;

	$err_no = count($messages);
	foreach($row as $fld_name => $fld_value) {
		$$fld_name = $fld_value;
	}
	$ret = eval( 'return '.$eval.';');
	// return null if there was any arror when evaluating 
	return ($err_no != count($messages) ? null : $ret);
}
//-----------------------------------------------------------------------------
//	Dumping data
//
function dump_data($file, $tname, &$dbf, &$pars) 
{

 global $import_cfg;
	
 $num_rec = $dbf->dbf_num_rec;

 $filter = $pars['filter'] =='' ? true : $pars['filter'];
 
 for ($i = 0; $i < $num_rec; $i++){

	$row = $dbf->getRowAssoc($i, $pars['getdeleted']);
	if (!$row) continue; // skip deleted rows
	// Convert source database record to mysql accepted format
	foreach($row as $key => $fld) {
		$val = $row[$key];
		switch ($dbf->dbf_names[$key]['type']) {
			case 'L':
				$row[$key] = $val == 'T' ? '1' : '0'; break;
			case 'D':
				$row[$key] = sprintf( "%04d-%02d-%02d", substr($val,0,4),
					substr($val,4,2), substr($val,6,2));
					 break;					
/* those field format can be accepted without changes
			case 'C':
			case 'M':
			case 'N':
			case 'F':
				break;
			default:
				display_error('Internal error: Unsupported field type on insert stage: '
					. $key . ':' . $dbf->dbf_names[$key]['type']);
				return 0;
*/		}
	}
	// skip filtered out rows
	$ret = eval_sandbox($filter, $row);
	if ($ret === null) {
		display_error(_('There were errors in filter expression for').' "'.
		 $tname .'". '._('Processing aborted.'));
		return 0;
	}
	if (!$ret) continue;

	$names = $values = array();
	$defs = $import_cfg['tables'][$tname]['fields'];
    foreach ($defs as $fname => $fldef) {
		$names[] = $fname;
		if (!isset($fldef['eval'])) {
			display_error("Internal error: No eval expression for field '$fname'.");
			return 0;
		}
	
		$val = eval_sandbox($fldef['eval'], $row);
		if ($val === null) {
			display_error(_('There were errors while evaluating expression for').' "'.
				$tname.'.'. $fname .'". '._('Processing aborted.'));
			return 0;
		}
		if (function_exists('mysql_real_escape_string')) {
			$values[] = "'" . mysql_real_escape_string($val) . "'";
		} else {
			$values[] = "'" . mysql_escape_string($val) . "'";
		}
	}	

	$line = 'INSERT INTO `' . TB_PREF . $tname .'` ';
	if (count($names) != count(get_table_struct($tname))) {
		$line .= '('.implode(',', $names) .') ';
	}
	$line .= 'VALUES('. implode(',', $values).');';

	fwrite($file, $line . "\n");
 } 
 return 1;
}
//------------------------------------------------------------------------------
//
//
function write_sql() 
{
	global $comp_path, $path_to_root, $type_trans, $import_cfg;
		
	$data_dir = $import_cfg['data_dir'];
	
	$sqlfile = $comp_path.'/'.user_company().'/backup/import_'.
		$data_dir.'.sql';

	@unlink($sqlfile); // not exactly needed but sometimes usefull
	$file = fopen($sqlfile, 'w');
	if(!$file) {
		display_error(_('Cannot open output script file'));
		return;
	}
    $timer = new timerClass();
    $timer ->start();
	$tables = $import_cfg['tables'];
	$err = 0;
	foreach($tables as $tname => $pars) {
		$fname = $pars['fname'];
    	$dbf = new dbf_class(IMPORT_DIR.'/'.$data_dir.'/'.$fname);

		if(isset($import_cfg['flush']) && $import_cfg['flush']==true) {

			fwrite($file, sprintf("--\n-- Table `%s` imported from `%s`.\n--\n", TB_PREF.$tname, $fname));
			fwrite($file, sprintf("DROP TABLE IF EXISTS `%s`;\n", TB_PREF.$tname));
			$result = db_query('SHOW CREATE TABLE ' . TB_PREF . $tname, 
				'could not retrieve create phrase');
			$row = db_fetch($result);
			fwrite($file, $row[1]."\n");
		}
		fwrite($file, sprintf("--\n-- Dumping data for table `%s`\n--\n", TB_PREF.$tname));	
		$err = !dump_data($file, $tname, $dbf, $pars);
		if ($err) break;
		 unset($dbf);
	}
	fclose($file);	
	if ($err)
		@unlink($sql_file);
	else {
		$endexct = $timer->end();
		display_notification(_("Import script has been generated: ")
			. $sqlfile . "(".($endexct/1000)." s)");
	}
}

?>